#!/usr/bin/perl

use strict;
use warnings;
use GD; # note: copyResampled works with GD 2>
use GDPlane;
use Data::Dumper;

#-----------------------------------------------------------------------------
# VARIABLES you might want to change
#-----------------------------------------------------------------------------
my $tex2im	= "/usr/local/bin/tex2im";
my $imageWidth	= 1024;
my $imageHeight	= 768;

my $horizStep	= 1;
my $numHorizStep= 6;
my $vertStep	= .5;
my $numVertStep	= 9;

my $planeX	= 10;
my $planeY	= 500;
my $planeWidth	= 450;
my $planeHeight	= 300;

#my @points = (	[0,3],
#		[1,2.5],
#		[1,3.5],
#		[2,1],
#		[2,3],
#		[4,1.5],
#		[4,2.5],
#		[5,1.5],
#		[5,2.5],
#		);
#
#my @centroids = (
#		[0,3],
#		[1,3.5],
#		[2,1],
#		);
my @points = (	[1,1],
		[1,3],
		[1,4],
		[2,1],
		[2,2],
		[5,3],
		[5,4],
		);

my @centroids = (
		[1,1],
		[1,4],
		[2,1],
		);

my @D; # distance matrix
my @G; # object clustering matrix

#-----------------------------------------------------------------------------
# And here comes the main app!
#-----------------------------------------------------------------------------

GD::Image->trueColor(1);

my $step = 0;
my $updated = 1;

while ($updated){
	my $img = new GD::Image( $imageWidth , $imageHeight );
	my $white = $img->colorAllocate( 255, 255, 255 );
	$img->fill(0,0,$white);	
	my $plane = new GDPlane (
				img => $img,
				hstp => $horizStep,
				vstp => $vertStep,
				numx => $numHorizStep,
				numy => $numVertStep,
				);
	
	$plane->draw($planeX, $planeY, $planeWidth, $planeHeight);
	
	# draw points and centroids
	addPoints ($plane, \@points);
	addCentroids ($plane, \@centroids);
	
	calculateD(\@D);
	calculateG(\@G, \@D);

	embedFormulas($img, $step, \@centroids, \@D, \@G);	
	saveImage($img,sprintf("%02d",$step));

	# note: updated is 1 if centroids change	
	$updated = updateCentroids(\@G, \@centroids);
	
	$step++;
}

sub CtoLatex{
	my ($centroids, $step) = @_; 
	my $latex = "";
	
	$latex .= '\begin{array}{c}'."\n"; 
	
	for (my $j = 0; $j<@$centroids; $j++){
		$latex .= sprintf ("c_{%d}^{%d} = (%.02f, %.02f) ", $j+1, $step, $$centroids[$j][0],$$centroids[$j][1]);
		$latex .= "\\\\\n";
	}
	$latex.="\\end{array}\n";
	return $latex;
}

sub embedFormulas{
	my ($img, $step, $C, $D, $G) = @_;

	# 1) Convert C, D and G to latex
	my $Ctex = CtoLatex($C,$step);
	my $Dtex = DtoLatex($D,$step);
	my $Gtex = GtoLatex($G,$step);

	# 2) Convert latex to images
	my $Cimg = tex2img($Ctex);
	my $Dimg = tex2img($Dtex);
	my $Gimg = tex2img($Gtex);
	
	# 3) Embed images inside main picture
	my ($wi, $hi) = $img->getBounds();
	my ($wc, $hc) = $Cimg->getBounds();
	my ($wd, $hd) = $Dimg->getBounds();
	my ($wg, $hg) = $Gimg->getBounds();

	my ($x, $y);
	my $freex =  $wi-$planeWidth-100;
	
	$x = $wi-$wc-40;
	$y = 40;
	$img->copy($Cimg, $x, $y, 0, 0, $wc, $hc);	

	$y += $hc+40;
	if ($wd>$freex){
		$x = $wi-$freex-40;
		$img->copyResampled($Dimg, $x, $y, 0, 0, $freex, $hd, $wd, $hd);
	}else{
		$x = $wi-$wd-40;
		$img->copy($Dimg, $x, $y, 0, 0, $wd, $hd);
	}	

	$y += $hd+40;
	if ($wg>$freex){
		$x = $wi-$freex-40;
		$img->copyResampled($Gimg, $x, $y, 0, 0, $freex, $hg, $wg, $hg);
	}else{
		$x = $wi-$wg-40;
		$img->copy($Gimg, $x, $y, 0, 0, $wg, $hg);
	}	
}

#-----------------------------------------------------------------------------

sub saveImage{
	my ($img, $suffix) = @_;
	open( PICT, ">fig$suffix.png" ) || die( "Cannot open picture: $!" );
	binmode( PICT );
	print( PICT $img->png() );
	close( PICT ) or die( "Can not close file: $!" );
}

#-----------------------------------------------------------------------------

sub tex2img{
	my ($latex) = @_;
	my $tmptex = "/tmp/__tex";
	my $tmpimg = "/tmp/__img";
	open( OUT, ">$tmptex") || die ("Cannot open temporary latex file: $!");
	print OUT $latex;
	close OUT;
	system ("$tex2im -o $tmpimg $tmptex");
	if ($? == -1){
		die "Failed to execute tex2im: $!\n";
	}
	my $image = GD::Image->new($tmpimg);
	return $image;
}

#-----------------------------------------------------------------------------

sub addPoints{
	my ($plane, $points) = @_;
	for (my $i = 0; $i<@$points; $i++){
		$plane->addPoint($$points[$i][0], $$points[$i][1]);
	}
}

#-----------------------------------------------------------------------------

sub addCentroids{
	my ($plane, $centroids) = @_;
	for (my $i = 0; $i<@$centroids; $i++){
		$plane->addCentroid($$centroids[$i][0], $$centroids[$i][1]);
	}
}

#-----------------------------------------------------------------------------

sub calculateD{
	my ($D) = @_;
	for (my $i = 0; $i<@points; $i++){
		for (my $j = 0; $j<@centroids; $j++){
			$$D[$i][$j]=sqrt((($points[$i][0] - $centroids[$j][0])**2) +
					 (($points[$i][1] - $centroids[$j][1])**2));
		}
	}

}

#-----------------------------------------------------------------------------

sub DtoLatex{
	my ($D, $step) = @_;
	my $latex="";

	$latex .= 'D^{'.$step.'} = \left[ \begin{array}{';
	for (my $i=0; $i<@points; $i++){
		$latex .= sprintf "c ";
	}
	$latex .= "}\n";
	for (my $j = 0; $j<@centroids; $j++){
		for (my $i = 0; $i<@points; $i++){
			$latex .= sprintf "%.2f ", $$D[$i][$j];
			$latex .= sprintf "& " if $i<@points-1;
		}
		$latex .= sprintf "\\\\ \n";
	}
	$latex .= '\end{array}\right]'."\n";
	return $latex;
}

#-----------------------------------------------------------------------------

sub GtoLatex{
	my ($G, $step) = @_;
	my $latex="";

	$latex .= 'G^{'.$step.'} = \left[ \begin{array}{';
	for (my $i=0; $i<@points; $i++){
		$latex .= sprintf "c ";
	}
	$latex .= "}\n";
	for (my $j = 0; $j<@centroids; $j++){
		for (my $i = 0; $i<@points; $i++){
			$latex .= sprintf "%.2f ", $$G[$i][$j];
			$latex .= sprintf "& " if $i<@points-1;
		}
		$latex .= sprintf "\\\\ \n";
	}
	$latex .= '\end{array}\right]'."\n";
	return $latex;
}

#-----------------------------------------------------------------------------

sub calculateG{
	my ($G, $D) = @_;
	for (my $i = 0; $i<@points; $i++){
		for (my $j = 0; $j<@centroids; $j++){
			$$G[$i][$j] = 0;
			if ($$D[$i][$j] == min($$D[$i])){
				$$G[$i][$j] = 1;
			}
		}
	}
}

#-----------------------------------------------------------------------------

sub updateCentroids{
	my ($G, $centroids) = @_;
	my $updated = 0;
	for (my $j = 0; $j<@$centroids; $j++){
		my $xsum = 0;
		my $ysum = 0;
		my $members = 0;
		for (my $i = 0; $i<@points; $i++){
			if ($$G[$i][$j]){
				$xsum += $points[$i][0];
				$ysum += $points[$i][1];
				$members++;
			}
		}
		$updated = 1 if (($$centroids[$j][0] != $xsum/$members) ||
				 ($$centroids[$j][1] != $ysum/$members));
		$$centroids[$j][0] = $xsum/$members;
		$$centroids[$j][1] = $ysum/$members;
	}
	return $updated;
}

#-----------------------------------------------------------------------------

sub min{
	my ($array) = shift;
	my $min = $$array[0];
	for (my $i=1; $i<@$array; $i++){
		if ($min > $$array[$i]){
			$min = $$array[$i];
		}
	}
	return $min;
}

#-----------------------------------------------------------------------------
